import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

public class Application {

	public static void main(String[] args) {
		BufferedReader input;
		File file = new File("imdb_simple.txt");
		String line;
		List<Movie> movies = new ArrayList<Movie>();
		Map<Actor, Double> actors = new HashMap<>();
		List<Entry<Actor, Double>> actorsLeaderboard;
		Map<Integer, Integer> years = new HashMap<>();
		List<Entry<Integer, Integer>> yearsLeaderboard;
		
		if (file.isFile()) {
			try {
				input = new BufferedReader(new FileReader(file));
				while ((line = input.readLine()) != null) {
					movies.add(parseMovie(line));
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		Collections.sort(movies, Collections.reverseOrder(new MovieComparator()));
		
		System.out.println("Ekstrahujemo podatke i stampamo sortirane filmove...\n");
		
		for(var movie : movies) {
			System.out.println(movie);
			Actor leadingKey = movie.getLeadingActor();
			Actor supportingKey = movie.getSupportingActor();
			Integer year = movie.getYear();
			if (actors.containsKey(leadingKey))
				actors.put(leadingKey, actors.get(leadingKey) + 1.0);
			else 
				actors.put(leadingKey, 1.0);
			if (actors.containsKey(supportingKey))
				actors.put(supportingKey, actors.get(supportingKey) + 0.4);
			else 
				actors.put(supportingKey, 0.4);
			if(years.containsKey(year))
				years.put(year, years.get(year) + 1);
			else
				years.put(year, 1);
		}
		
		actorsLeaderboard = new ArrayList<>(actors.entrySet());
		actorsLeaderboard.sort(Entry.comparingByValue(Comparator.reverseOrder()));
		
		System.out.println("Sortiramo glumce...\n");
		
		for(var actorEntry: actorsLeaderboard) {
			System.out.println(String.format("%-35s%-5s\n", actorEntry.getKey(), actorEntry.getValue()));
		}
		
		yearsLeaderboard = new ArrayList<>(years.entrySet());
		yearsLeaderboard.sort(Entry.comparingByValue(Comparator.reverseOrder()));
		
		System.out.println("Sortiramo godine...\n");
		
		for(var yearEntry: yearsLeaderboard) {
			System.out.println(String.format("%-6s%-2s\n", yearEntry.getKey(), yearEntry.getValue()));
		}
	}
	
	public static Movie parseMovie(String line) {
		String args[] = line.split("#");
		return new Movie(args);
	}

}
